<?php

namespace App\Http\Controllers\Admin;

use App\Http\Traits\MessageStatusTrait;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\BusinessOrderType;
use App\Models\BusinessCategory;
use App\Models\OrderType;
use App\Models\Business;
use App\Models\Category;
use App\Models\Country;
use App\Models\Owner;
use App\Models\State;
use App\Models\City;
use Validator;
use Auth;
use Hash;
use File;

class BusinessController extends Controller
{
    use MessageStatusTrait;

    # Bind location
    protected $view = 'admin.business.';
    protected $type = 'Owner ';


    # Bind owner
    protected $city;
    protected $owner;
    protected $state;
    protected $country;
    protected $category;
    protected $business;
    protected $orderType;
    protected $businessCategory;
    protected $businessOrderType;
  
    /**
     * default constructor
     * @param
     * @return
     */
    function __construct(
                          City       $city,
                          Owner      $owner,
                          State      $state,
                          Country    $country,
                          Category   $category,
                          Business  $business,
                          BusinessCategory  $businessCategory,
                          BusinessOrderType  $businessOrderType,
                          OrderType  $orderType
                        )
                        {
                            $this->city      = $city;
                            $this->state     = $state;
                            $this->owner     = $owner;
                            $this->country   = $country;
                            $this->category  = $category;
                            $this->business = $business;
                            $this->businessCategory = $businessCategory;
                            $this->businessOrderType = $businessOrderType;
                            $this->orderType = $orderType;
                            $this->page      = config('paginate.pagination');
                        }



   
    #business page
    public function index(Request $request){
        $query = $this->business;
      
        $business_list = $query->orderBy('id','desc')->paginate($this->page);

        return view($this->view.'index')->with([
                                                 'business_list'  => $business_list ?? [],
                                               ]);
    }



     #owner add page
    public function create(Request $request){
          $owner_list = $this->owner->where('status',1)->orderBy('id','desc')->get(); 
          $category_list = $this->category->where('status',1)->orderBy('id','desc')->get(); 
          $orderType_list = $this->orderType->where('status',1)->orderBy('id','desc')->get(); 

          return view($this->view.'add')->with([
                                                 'owner_list'     => $owner_list,
                                                 'category_list'  => $category_list,
                                                 'orderType_list' => $orderType_list,
                                               ]);
    }



    /**
      * add category
      * @param Illuminate\Http\Request; 
      * @return Illuminate\Http\Response;
      */
     public function store(Request $request)
     {
      $data = [
                'owner_id'         => 'required',
                'name'             => 'required',
                'mobile'           => 'required',
                'email'            => 'required',
                'address'          => 'required',
                'latitude'         => 'required',
                'longitude'        => 'required',
                'username'         => 'required',
                'password'         => 'required',
                'image'            => 'required',
                'description'      => 'required',
                'category_id'      => 'required',
                'order_type_id'     => 'required',
             ];

            # validation check
            $validator = Validator::make($request->all() , $data);
            if($validator->fails())
             {
                return response()->json([
                                          'responseCode'    =>  $this->errorStatus, 
                                          'responseMessage' => 'Required field is missing.'
                                        ]);
             }

            try { 

                   #check the requested email already exist or not
                    $checkEmail = $this->business
                                       ->where('email', $request->email)
                                       ->first();

                   
                   if($checkEmail)
                    {   
                      return response()->json([
                                                'responseCode'    =>  $this->errorStatus, 
                                                'responseMessage' =>'Sorry, this email already exist.'
                                              ]);  
                    }

                     #check the requested username already exist or not
                    $checkusername = $this->business
                                       ->where('username', $request->username)
                                       ->first();

                   
                   if($checkusername)
                    {   
                      return response()->json([
                                                'responseCode'    =>  $this->errorStatus, 
                                                'responseMessage' =>'Sorry, this username already exist.'
                                              ]);  
                    }

                 #upload image
                if ($request->hasfile('image'))
                {
                    $file = $request->file('image');
                    $extension = $file->getClientOriginalExtension(); // getting image extension
                    $filename =((string)(microtime(true)*10000)).'.'.$extension;
                    $file->move(public_path('images/business/'), $filename);
                    $Image='images/business/'.$filename;
                }else{
                    $Image = null;
                }
                
                # request param
                $arrayData = [
                               'owner_id'            => $request->owner_id ?? null,
                               'name'                => $request->name ?? null,
                               'email'               => $request->email ?? null,
                               'mobile'              => $request->mobile ?? null,
                               'alt_mobile'          => $request->alt_mobile ?? null,
                               'address'             => $request->address ?? null,
                               'latitude'            => $request->latitude ?? null,
                               'longitude'           => $request->longitude ?? null,
                               'username'            => $request->username ?? null,
                               'password'            => Hash::make($request->password),
                               'website_url'         => $request->website_url ?? null,
                               'description'         => $request->description ?? null,
                               'openTimeMonday'      => $request->openTimeMonday ?? null,
                               'closeTimeMonday'     => $request->closeTimeMonday ?? null,
                               'openTimeTuesday'     => $request->openTimeTuesday ?? null,
                               'closeTimeTuesday'    => $request->closeTimeTuesday ?? null,
                               'openTimeWednesday'   => $request->openTimeWednesday ?? null,
                               'closeTimeWednesday'  => $request->closeTimeWednesday ?? null,
                               'openTimeThursday'    => $request->openTimeThursday ?? null,
                               'closeTimeThursday'   => $request->closeTimeThursday ?? null,
                               'openTimeFriday'      => $request->openTimeFriday ?? null,
                               'closeTimeFriday'     => $request->closeTimeFriday ?? null,
                               'openTimeSaturday'    => $request->openTimeSaturday ?? null,
                               'closeTimeSaturday'   => $request->closeTimeSaturday ?? null,
                               'openTimeSunday'      => $request->openTimeSunday ?? null,
                               'closeTimeSunday'     => $request->closeTimeSunday ?? null,
                               'status'              => 1,
                               'logo'                => $Image ?? null,
                             ];     

                     
                #store
                $business_store = $this->business->create($arrayData); 


                foreach($request->category_id as $category){
                    # request param
                    $arrayData = [
                                   'business_id'        => $business_store->id,
                                   'category_id'         => $category,
                                 ]; 

                    #store
                    $store = $this->businessCategory->create($arrayData);
                }


                foreach($request->order_type_id as $orderType){
                    # request param
                    $arrayData = [
                                   'business_id'        => $business_store->id,
                                   'order_type_id'       => $orderType,
                                 ]; 

                    #store
                    $store = $this->businessOrderType->create($arrayData);
                }

                               
                return response()->json([
                                          'responseCode'    =>$this->successStatus, 
                                          'responseMessage' => 'Business added successfully.'
                                        ]);                      

            } catch (Exception $e) {
                return response()->json([
                                          'responseCode'    =>$this->errorStatus, 
                                          'responseMessage' => 'Something went wrong.'
                                        ]);  
            }
     }


    /**
     * edit page
     * @param Illuminate\Http\Request;
     * @return Illuminate\Http\Response;
     */
    public function update($id)
    {  
         # Fetch business by id
         $business = $this->business
                           ->where('id', $id)
                           ->first();

          $category_list = $this->category->where('status',1)->orderBy('id','desc')->get(); 
          $orderType_list = $this->orderType->where('status',1)->orderBy('id','desc')->get();

          $categoryId = $this->businessCategory->where('business_id',$id)->pluck('category_id')->toArray(); 
          $orderTypeId = $this->businessOrderType->where('business_id',$id)->pluck('order_type_id')->toArray(); 
          $owner_list = $this->owner->where('status',1)->orderBy('id','desc')->get();

        # code...
        return view($this->view . 'edit')->with([
                                                  'business'       => $business,
                                                  'categoryId'     => $categoryId,
                                                  'orderTypeId'    => $orderTypeId,
                                                  'owner_list'     => $owner_list,
                                                  'category_list'  => $category_list,
                                                  'orderType_list' => $orderType_list,
                                               ]);
    }



    /**
      * edit category
      * @param Illuminate\Http\Request; 
      * @return Illuminate\Http\Response;
      */
     public function edit(Request $request)
     {
          $data = [
                'business_id'      => 'required',
                'owner_id'         => 'required',
                'name'             => 'required',
                'mobile'           => 'required',
                'email'            => 'required',
                'address'          => 'required',
                'latitude'         => 'required',
                'longitude'        => 'required',
                'username'         => 'required',
                'description'      => 'required',
                'category_id'      => 'required',
                'order_type_id'    => 'required',
             ];

            # validation check
            $validator = Validator::make($request->all() , $data);
            if($validator->fails())
             {
                return response()->json([
                                          'responseCode'    =>  $this->errorStatus, 
                                          'responseMessage' => 'Required field is missing.'
                                        ]);
             }

            try {

                     #check the requested email already exist or not
                    $checkEmail = $this->business
                                       ->where('email', $request->email)
                                       ->where('id','!=' ,$request->business_id)
                                       ->first();

                   
                   if($checkEmail)
                    {   
                      return response()->json([
                                                'responseCode'    =>  $this->errorStatus, 
                                                'responseMessage' =>'Sorry, this email already exist.'
                                              ]);  
                    }

                     #check the requested username already exist or not
                    $checkusername = $this->business
                                       ->where('username', $request->username)
                                       ->where('id','!=' ,$request->business_id)
                                       ->first();

                   
                   if($checkusername)
                    {   
                      return response()->json([
                                                'responseCode'    =>  $this->errorStatus, 
                                                'responseMessage' =>'Sorry, this username already exist.'
                                              ]);  
                    }
              
               $query = $this->business
                             ->where('id', $request->business_id)
                             ->first();
                              
                #upload image
                if ($request->hasfile('image'))
                {
                    $file = $request->file('image');
                    $extension = $file->getClientOriginalExtension(); // getting image extension
                    $filename =((string)(microtime(true)*10000)).'.'.$extension;
                    $file->move(public_path('images/business/'), $filename);
                    $Image='images/business/'.$filename;
                }else{
                    $Image = $query->logo ?? null;
                }
                
                # request param
                # request param
                $arrayData = [
                               'owner_id'            => $request->owner_id ?? null,
                               'name'                => $request->name ?? null,
                               'email'               => $request->email ?? null,
                               'mobile'              => $request->mobile ?? null,
                               'alt_mobile'          => $request->alt_mobile ?? null,
                               'address'             => $request->address ?? null,
                               'latitude'            => $request->latitude ?? null,
                               'longitude'           => $request->longitude ?? null,
                               'username'            => $request->username ?? null,
                               'website_url'         => $request->website_url ?? null,
                               'description'         => $request->description ?? null,
                               'openTimeMonday'      => $request->openTimeMonday ?? null,
                               'closeTimeMonday'     => $request->closeTimeMonday ?? null,
                               'openTimeTuesday'     => $request->openTimeTuesday ?? null,
                               'closeTimeTuesday'    => $request->closeTimeTuesday ?? null,
                               'openTimeWednesday'   => $request->openTimeWednesday ?? null,
                               'closeTimeWednesday'  => $request->closeTimeWednesday ?? null,
                               'openTimeThursday'    => $request->openTimeThursday ?? null,
                               'closeTimeThursday'   => $request->closeTimeThursday ?? null,
                               'openTimeFriday'      => $request->openTimeFriday ?? null,
                               'closeTimeFriday'     => $request->closeTimeFriday ?? null,
                               'openTimeSaturday'    => $request->openTimeSaturday ?? null,
                               'closeTimeSaturday'   => $request->closeTimeSaturday ?? null,
                               'openTimeSunday'      => $request->openTimeSunday ?? null,
                               'closeTimeSunday'     => $request->closeTimeSunday ?? null,
                               'status'              => 1,
                               'logo'                => $Image ?? null,
                             ];     

                     
                #update
                $update = $query->update($arrayData); 

                $this->businessCategory->where('business_id',$request->business_id)->delete();
                foreach($request->category_id as $category){
                    # request param
                    $arrayData = [
                                   'business_id'        => $request->business_id,
                                   'category_id'         => $category,
                                 ]; 

                    #store
                    $store = $this->businessCategory->create($arrayData);
                }

                $this->businessOrderType->where('business_id',$request->business_id)->delete();
                foreach($request->order_type_id as $orderType){
                    # request param
                    $arrayData = [
                                   'business_id'        => $request->business_id,
                                   'order_type_id'       => $orderType,
                                 ]; 

                    #store
                    $store = $this->businessOrderType->create($arrayData);
                } 

                return response()->json([
                                          'responseCode'    =>$this->successStatus, 
                                          'responseMessage' => 'Business Updated Successfully.'
                                        ]);                      

            } catch (Exception $e) {
                return response()->json([
                                          'responseCode'    =>$this->errorStatus, 
                                          'responseMessage' => 'Something went wrong.'
                                        ]);  
            }
     }




   
}
