<?php

namespace App\Http\Controllers\Admin;

use App\Http\Traits\MessageStatusTrait;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\OrderType;
use Validator;
use Auth;
use Hash;
use File;

class OrderTypeController extends Controller
{
    use MessageStatusTrait;

    # Bind location
    protected $view = 'admin.order-type.';
    protected $type = 'Order Type ';


    # Bind OrderType
    protected $orderType;
  
    /**
     * default constructor
     * @param
     * @return
     */
    function __construct(
                          OrderType $orderType
                        )
                        {
                            $this->orderType   = $orderType;
                            $this->page = config('paginate.pagination');
                        }



   
    #orderType page
    public function index(Request $request){
        $query = $this->orderType;
      
        $ordertype_list = $query->orderBy('id','desc')->paginate($this->page);
        return view($this->view.'index')->with([
                                                 'ordertype_list'  => $ordertype_list ?? [],
                                               ]);
    }



     #orderType add page
    public function create(Request $request){
          return view($this->view.'add');
    }



    /**
      * add orderType
      * @param Illuminate\Http\Request; 
      * @return Illuminate\Http\Response;
      */
     public function store(Request $request)
     {
      $data = [
                'name'           => 'required',
                'commission'     => 'required',
             ];

            # validation check
            $validator = Validator::make($request->all() , $data);
            if($validator->fails())
             {
                return response()->json([
                                          'responseCode'    =>  $this->errorStatus, 
                                          'responseMessage' => 'Required field is missing.'
                                        ]);
             }

            try { 
                
                # request param
                $arrayData = [
                               'name'            => $request->name ?? null,
                               'commission'      => $request->commission ?? null,
                               'status'          => 1
                             ];     

                #store
                $storeorderType = $this->orderType->create($arrayData); 

                               
                return response()->json([
                                          'responseCode'    =>$this->successStatus, 
                                          'responseMessage' => 'Order Type added successfully.'
                                        ]);                      

            } catch (Exception $e) {
                return response()->json([
                                          'responseCode'    =>$this->errorStatus, 
                                          'responseMessage' => 'Something went wrong.'
                                        ]);  
            }
     }


    /**
     * edit page
     * @param Illuminate\Http\Request;
     * @return Illuminate\Http\Response;
     */
    public function update($id)
    {  
        # Fetch orderType by id
         $orderType = $this->orderType
                           ->where('id', $id)
                           ->first();
        # code...
        return view($this->view . 'edit')->with(['orderType' => $orderType]);
    }



    /**
      * edit orderType
      * @param Illuminate\Http\Request; 
      * @return Illuminate\Http\Response;
      */
     public function edit(Request $request)
     {
        $data = [
                'name'           => 'required',
                'commission'     => 'required',
             ];

            # validation check
            $validator = Validator::make($request->all() , $data);
            if($validator->fails())
             {
                return response()->json([
                                          'responseCode'    =>  $this->errorStatus, 
                                          'responseMessage' => 'Required field is missing.'
                                        ]);
             }
             
            try {
              
               $query = $this->orderType
                             ->where('id', $request->id)
                             ->first();
                              
               # request param
                $arrayData = [
                               'name'            => $request->name ?? null,
                               'commission'      => $request->commission ?? null,
                             ];   

                #update
                $updateorderType = $query->update($arrayData); 

                return response()->json([
                                          'responseCode'    =>$this->successStatus, 
                                          'responseMessage' => 'Order Type Updated Successfully.'
                                        ]);                      

            } catch (Exception $e) {
                return response()->json([
                                          'responseCode'    =>$this->errorStatus, 
                                          'responseMessage' => 'Something went wrong.'
                                        ]);  
            }
     }


   /**
     * delete orderType
     * @param $id
     * @return \Illuminate\Http\Response
     */
    public function delete($id)
    {
        $query = $this->orderType;

        # delete orderType by id
        $query->where('id', $id)->delete();

        # return success
        return [$this->successKey => $this->successStatus, $this->messageKey => $this->deleteMessage($this->type) ];
    }


   
}
