<?php

namespace App\Http\Controllers\Admin;

use App\Http\Traits\MessageStatusTrait;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use Validator;
use Auth;
use Hash;
use File;

class SubCategoryController extends Controller
{
    use MessageStatusTrait;

    # Bind location
    protected $view = 'admin.sub-category.';
    protected $type = 'SubCategory ';


    # Bind category
    protected $category;
  
    /**
     * default constructor
     * @param
     * @return
     */
    function __construct(
                          Category $category
                        )
                        {
                            $this->category   = $category;
                            $this->page = config('paginate.pagination');
                        }



   
    #subcategory page
    public function index(Request $request){
        $query = $this->category;
      
        $category_list = $query->where('parent_id','!=',0)
                               ->orderBy('sequence_number','asc')
                               ->paginate($this->page);

        return view($this->view.'index')->with([
                                                 'category_list'  => $category_list ?? [],
                                               ]);
    }



     #subcategory add page
    public function create(Request $request){
          $category_list = $this->category
                                ->orderBy('sequence_number','asc')
                                ->get();

          return view($this->view.'add')->with(['category_list' => $category_list]);
    }



    /**
      * add category
      * @param Illuminate\Http\Request; 
      * @return Illuminate\Http\Response;
      */
     public function store(Request $request)
     {
      $data = [
                'name'             => 'required',
                'image'            => 'required',
                'sequence_number'  => 'required',
                'details'          => 'required',
                'type'             => 'required',
                'parent_id'        => 'required',
             ];

            # validation check
            $validator = Validator::make($request->all() , $data);
            if($validator->fails())
             {
                return response()->json([
                                          'responseCode'    =>  $this->errorStatus, 
                                          'responseMessage' => 'Required field is missing.'
                                        ]);
             }

            try { 

                 #upload image
                if ($request->hasfile('image'))
                {
                    $file = $request->file('image');
                    $extension = $file->getClientOriginalExtension(); // getting image extension
                    $filename =((string)(microtime(true)*10000)).'.'.$extension;
                    $file->move(public_path('images/category/'), $filename);
                    $catImage='images/category/'.$filename;
                }else{
                    $catImage = null;
                }
                
                # request param
                $arrayData = [
                               'name'            => $request->name ?? null,
                               'sequence_number' => $request->sequence_number ?? null,
                               'details'         => $request->details ?? null,
                               'type'            => $request->type ?? null,
                               'parent_id'       => $request->parent_id ?? null,
                               'status'          => 1,
                               'image'           => $catImage ?? null,
                             ];     

                #store
                $store = $this->category->create($arrayData); 

                               
                return response()->json([
                                          'responseCode'    =>$this->successStatus, 
                                          'responseMessage' => 'SubCategory added successfully.'
                                        ]);                      

            } catch (Exception $e) {
                return response()->json([
                                          'responseCode'    =>$this->errorStatus, 
                                          'responseMessage' => 'Something went wrong.'
                                        ]);  
            }
     }


    /**
     * edit page
     * @param Illuminate\Http\Request;
     * @return Illuminate\Http\Response;
     */
    public function update($id)
    {  
        $category_list = $this->category
                              ->orderBy('sequence_number','asc')
                              ->get();

        # Fetch category by id
         $category = $this->category
                           ->where('id', $id)
                           ->first();
        # code...
        return view($this->view . 'edit')->with([
                                                  'category'      => $category,
                                                  'category_list' => $category_list
                                               ]);
    }



    /**
      * edit category
      * @param Illuminate\Http\Request; 
      * @return Illuminate\Http\Response;
      */
     public function edit(Request $request)
     {
          $data = [
                'name'             => 'required',
                'sequence_number'  => 'required',
                'details'          => 'required',
                'type'             => 'required',
                'parent_id'        => 'required',
             ];

            # validation check
            $validator = Validator::make($request->all() , $data);
            if($validator->fails())
             {
                return response()->json([
                                          'responseCode'    =>  $this->errorStatus, 
                                          'responseMessage' => 'Required field is missing.'
                                        ]);
             }

            try {
              
               $query = $this->category
                             ->where('id', $request->id)
                             ->first();
                              
                #upload image
                if ($request->hasfile('image'))
                {
                    $file = $request->file('image');
                    $extension = $file->getClientOriginalExtension(); // getting image extension
                    $filename =((string)(microtime(true)*10000)).'.'.$extension;
                    $file->move(public_path('images/category/'), $filename);
                    $catImage='images/category/'.$filename;
                }else{
                    $catImage = $query->image ?? null;
                }
                
                # request param
                $arrayData = [
                               'name'            => $request->name ?? null,
                               'sequence_number' => $request->sequence_number ?? null,
                               'details'         => $request->details ?? null,
                               'type'            => $request->type ?? null,
                               'image'           => $catImage ?? null,
                               'parent_id'       => $request->parent_id ?? null,
                             ];       

                #update
                $update = $query->update($arrayData); 

                return response()->json([
                                          'responseCode'    =>$this->successStatus, 
                                          'responseMessage' => 'SubCategory Updated Successfully.'
                                        ]);                      

            } catch (Exception $e) {
                return response()->json([
                                          'responseCode'    =>$this->errorStatus, 
                                          'responseMessage' => 'Something went wrong.'
                                        ]);  
            }
     }



   
}
